@@ -2,12 +2,12 @@
 **  CWParser.m
 **
 **  Copyright (c) 2001-2007 Ludovic Marcotte
-**  Copyright (C) 2013-2017 The GNUstep Team
+**  Copyright (C) 2013-2020 The GNUstep Team
 **
 **  Author: Ludovic Marcotte <ludovic@Sophos.ca>
 **          Riccardo Mottola <rm@gnu.org>
 **          Sebastian Reitenbach
- **         German Arias
+**          German Arias
 **
 **  This library is free software; you can redistribute it and/or
 **  modify it under the terms of the GNU Lesser General Public
@@ -96,10 +96,10 @@ static struct _timezone {
 //
 //
 //
-int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned char *word)
+int next_word(unsigned char *buf, NSUInteger start, NSUInteger len, unsigned char *word)
 {
   unsigned char *p;
-  int i;
+  NSUInteger i;
 
   for (p = buf+start, i = start; (isspace(*p) || *p == ','); ++p, ++i);
   
@@ -113,7 +113,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
   
   *word = '\0';
 
-  return p-buf-start;
+  return (int)((NSUInteger)(p-buf)-start);
 }
 
 //
@@ -263,7 +263,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 {
   NSRange aRange;
   NSData *aData;
-  int x;
+  NSUInteger x;
 
   if ([theLine length] <= 14)
     {
@@ -272,10 +272,15 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
     }
 
   aData = [[theLine subdataFromIndex: 13] dataByTrimmingWhiteSpaces];
+  if (aData == nil || [aData length] == 0)
+    {
+      [thePart setContentType: @"text/plain"];
+      return;
+    }
 
   // We first skip the parameters, if we need to
   x = [aData indexOfCharacter: ';'];
-  if (x > 0)
+  if (x != NSNotFound && x > 0)
     {
       aData = [aData subdataToIndex: x];
     } 
@@ -283,7 +288,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
   // We see if there's a subtype specified for text, if none was specified, we append "/plain"
   x = [aData indexOfCharacter: '/'];
 
-  if (x < 0 && [aData hasCaseInsensitiveCPrefix: "text"])
+  if (x == NSNotFound && [aData hasCaseInsensitiveCPrefix: "text"])
     {
       [thePart setContentType: [[[aData asciiString] stringByAppendingString: @"/plain"] lowercaseString]];
     }
@@ -361,8 +366,10 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
   if ([theLine length] > 6)
     {
       NSData *aData;
-
-      int month, day, year, hours, mins, secs, tz, i, j, len, tot, s;
+      NSUInteger tot;
+      int month, day, year, hours, mins, secs, tz;
+      int len, s;
+      unsigned i, j;
       unsigned char *bytes, *word;
 
       aData = [theLine subdataFromIndex: 6];
@@ -419,7 +426,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 	  //NSLog(@"UNIX DATE");
 	  
 	  // We skip the first word, no need for it.
-	  i += len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
+	  i += (unsigned)len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
 	}
 
       
@@ -434,13 +441,13 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
       //printf("len = %d |%s| day = %d\n", len, word, day);
       
       // We get the month name and we convert it.
-      i += len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
+      i += (unsigned)len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
       
       for (j = 0; j < 12; j++)
 	{
 	  if (strncasecmp((const char*)word, month_name[j], 3) == 0)
 	    {
-	      month = j+1;
+	      month = (int)j+1;
 	    }
 	}
       
@@ -449,7 +456,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
       //printf("len = %d |%s| month = %d\n", len, word, month);
       
       // We get the year.
-      i += len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
+      i += (unsigned)len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
       year = atoi((const char*)word);
       
       if (year < 70) year += 2000;
@@ -458,12 +465,12 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
       //printf("len = %d |%s| year = %d\n", len, word, year);
       
       // We parse the time using the hh:mm:ss format.
-      i += len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
+      i += (unsigned)len+1; len = next_word(bytes, i, tot, word); if (len <= 0) { free(word); return; }
       sscanf((const char*)word, "%d:%d:%d", &hours, &mins, &secs);
       //printf("len = %d |%s| %d:%d:%d\n", len, word, hours, mins, secs);
       
       // We parse the timezone.
-      i += len+1; len = next_word(bytes, i, tot, word); 
+      i += (unsigned)len+1; len = next_word(bytes, i, tot, word);
       
       if (len <= 0)
 	{
@@ -481,7 +488,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 	      p++;
 	    }
 	  
-	  len = strlen((const char*)p);
+	  len = (int)strlen((const char*)p);
 	  
 	  if (isdigit(*p))
 	    {
@@ -508,11 +515,11 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 	}
       
       [theMessage setReceivedDate: [NSCalendarDate dateWithYear: year
-						   month: month
-						   day: day
-						   hour: hours
-						   minute: mins
-						   second: secs
+						   month: (NSUInteger) month
+						   day: (NSUInteger)day
+						   hour: (NSUInteger) hours
+						   minute: (NSUInteger) mins
+						   second: (NSUInteger) secs
 						   timeZone: [NSTimeZone timeZoneForSecondsFromGMT: tz]]];
       free(word);
     }
@@ -533,7 +540,6 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
   unsigned char *bytes;
   BOOL b;
 
-  len = 0;
   if (theBOOL)
     {
       aData = theLine;
@@ -559,6 +565,10 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 	case PantomimeResentToRecipient:
 	  len = 11;
 	  break;
+        default:
+          NSDebugLog(@"CWParser parseDestination: forType: unknown type!");
+          len = 0;
+          break;
 	}
 
       // We skip over emtpy headers.
@@ -601,9 +611,12 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 				initWithString: [CWMIMEUtility decodeHeader: headerData
 							       charset: [theMessage defaultCharset]]];
  
-	  [anInternetAddress setType: theType];
-	  [theMessage addRecipient: anInternetAddress];
-	  RELEASE(anInternetAddress);
+	  if (anInternetAddress != nil)
+            {
+              [anInternetAddress setType: theType];
+	      [theMessage addRecipient: anInternetAddress];
+	      RELEASE(anInternetAddress);
+            }
 	  x = i+1;
 	}
       
@@ -654,7 +667,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 		      quick: (BOOL) theBOOL
 {
   NSData *aData;
-  int x, y;
+  NSUInteger x, y;
   
   if (theBOOL)
     {
@@ -677,11 +690,11 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
   x = [aData indexOfCharacter: ';'];
   y = [aData indexOfCharacter: ' '];
 
-  if (x > 0 && x < y)
+  if (x != NSNotFound && x > 0 && x < y)
     {
       aData = [aData subdataToIndex: x];
     }
-  else if (y > 0)
+  else if (y != NSNotFound && y > 0)
     {
       aData = [aData subdataToIndex: y];
     }
@@ -753,7 +766,7 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
       NSArray *allReferences;
       NSData *aReference;
       NSString *aString;
-      int i, count;
+      NSUInteger i, count;
       
       allReferences = [aData componentsSeparatedByCString: " "];
       count = [allReferences count];
@@ -795,7 +808,8 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
       NSData *aData;
 
       unsigned char *bytes;
-      int i, len, s_len, x, y;
+      NSUInteger i, len;
+      int s_len, x, y;
       BOOL b;
       
       aMutableArray = [[NSMutableArray alloc] init];
@@ -828,9 +842,11 @@ int next_word(unsigned char *buf, unsigned int start, unsigned int len, unsigned
 	      anInternetAddress = [[CWInternetAddress alloc]
 				    initWithString: [CWMIMEUtility decodeHeader: [[aData subdataWithRange: NSMakeRange(x, s_len)] dataByTrimmingWhiteSpaces]
 								   charset: [theMessage defaultCharset]]];
-	      
-	      [aMutableArray addObject: anInternetAddress];
-	      RELEASE(anInternetAddress);
+	      if (anInternetAddress != nil)
+                {
+	          [aMutableArray addObject: anInternetAddress];
+	          RELEASE(anInternetAddress);
+                }
 	      x = y+1;
 	    }
 	  
