@@ -2,7 +2,7 @@
 **  NSData+Extensions.m
 **
 **  Copyright (c) 2001-2007 Ludovic Marcotte
-**  Copyright (C) 2014-2017 Riccardo Mottola
+**  Copyright (C) 2014-2022 Riccardo Mottola
 **
 **  Author: Ludovic Marcotte <ludovic@Sophos.ca>
 **          Riccardo Mottola <rm@gnu.org>
@@ -21,13 +21,13 @@
 ** along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#import "Pantomime/NSData+Extensions.h"
+#import <Pantomime/NSData+Extensions.h>
 
 #import <Foundation/NSArray.h>
 #import <Foundation/NSException.h>
 #import <Foundation/NSString.h>
 
-#import "Pantomime/CWConstants.h"
+#import <Pantomime/CWConstants.h>
 
 #include <stdlib.h>
 #include <string.h>
@@ -35,8 +35,8 @@
 //
 // C functions and constants
 //
-int getValue(char c);
-void nb64ChunkFor3Characters(char *buf, const char *inBuf, int numChars);
+int getValue(const unsigned char c);
+void nb64ChunkFor3Characters(char *buf, const char *inBuf, unsigned numChars);
 
 static const char basis_64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 static const char *hexDigit = "0123456789ABCDEF";
@@ -57,10 +57,11 @@ static const char *hexDigit = "0123456789ABCDEF";
 - (NSData *) decodeBase64
 {
   NSUInteger data_len;
-  NSUInteger i, j;
-  NSInteger length;
+  NSUInteger i;
+  NSUInteger length;
   NSUInteger pad;
-  int rawIndex, block;
+  unsigned rawIndex;
+  int block;
   const unsigned char *bytes;
   char *raw;
 
@@ -77,14 +78,30 @@ static const char *hexDigit = "0123456789ABCDEF";
     {
       pad++;
     }
+
+  // This could happen for broken encoded base64 content such as the following example:
+  // ------=_NextPart_KO_X1098V29876N91O412QM815
+  // Content-Type: text/plain; charset=UTF-8
+  // Content-Disposition: attachment; filename="teste.txt"
+  // Content-Transfer-Encoding: base64
+  //
+  // ====
+  //
+  // we want to test (data_len * 6 / 8 - pad) < 0 but are using unisgned values, so we reverse it
+  if ((data_len * 6 / 8) < pad)
+    {
+      return [NSData data];
+    }
   
   length = data_len * 6 / 8 - pad;
   
-  raw = (char *)malloc(length);
+  raw = (char *)malloc((size_t)length);
   rawIndex = 0;
 
   for (i = 0; i < data_len; i += 4)
     {
+      NSUInteger j;
+
       block = (getValue(bytes[i]) << 18) +
 	(getValue(bytes[i+1]) << 12) +
 	(getValue(bytes[i+2]) << 6) +
@@ -97,20 +114,6 @@ static const char *hexDigit = "0123456789ABCDEF";
 
       rawIndex += 3;
     }
-  
-  // This could happen for broken encoded base64 content such as the following example:
-  // ------=_NextPart_KO_X1098V29876N91O412QM815
-  // Content-Type: text/plain; charset=UTF-8
-  // Content-Disposition: attachment; filename="teste.txt"
-  // Content-Transfer-Encoding: base64
-  //
-  // ====
-  //
-  if (length < 0)
-    {
-      free(raw);
-      return [NSData data];
-    }
 
   return AUTORELEASE([[NSData alloc] initWithBytesNoCopy: raw  length: length]);
 }
@@ -119,30 +122,31 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 //
-- (NSData *) encodeBase64WithLineLength: (int) theLength
+- (NSData *) encodeBase64WithLineLength: (NSUInteger) theLength
 {
   const char *inBytes = [self bytes];
   const char *inBytesPtr = inBytes;
-  NSInteger inLength = [self length];
+  NSUInteger inLength = [self length];
+  NSUInteger i;
+  char *outBytes;
+  char *outBytesPtr;
 
-  char *outBytes = malloc(sizeof(char)*inLength*2);
-  char *outBytesPtr = outBytes;
-
-  int numWordsPerLine = theLength/4;
-  int wordCounter = 0;
+  NSUInteger numWordsPerLine = theLength/4;
+  NSUInteger wordCounter = 0;
 
+  outBytes = malloc(sizeof(char)*(size_t)inLength*2);
+  outBytesPtr = outBytes;
   // We memset 0 our buffer so with are sure to not have
   // any garbage in it.
-  memset(outBytes, 0, sizeof(char)*inLength*2);
+  memset(outBytes, 0, sizeof(char)*(size_t)inLength*2);
 
-  while (inLength > 0)
+  for (i = 0; i < inLength; i += 3)
     {
-      nb64ChunkFor3Characters(outBytesPtr, inBytesPtr, inLength);
+      nb64ChunkFor3Characters(outBytesPtr, inBytesPtr, (unsigned)(inLength-i));
       outBytesPtr += 4;
       inBytesPtr += 3;
-      inLength -= 3;
 
-      wordCounter ++;
+      wordCounter++;
 
       if (theLength && wordCounter == numWordsPerLine)
 	{
@@ -152,7 +156,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 	}
     }
 
-  return AUTORELEASE([[NSData alloc] initWithBytesNoCopy: outBytes length: (outBytesPtr-outBytes)]);
+  return AUTORELEASE([[NSData alloc] initWithBytesNoCopy: outBytes length: (NSUInteger)(outBytesPtr-outBytes)]);
 }
 
 
@@ -282,7 +286,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 //
-- (NSData *) encodeQuotedPrintableWithLineLength: (int) theLength
+- (NSData *) encodeQuotedPrintableWithLineLength: (NSUInteger) theLength
 					inHeader: (BOOL) aBOOL
 {
   NSMutableData *aMutableData;
@@ -370,7 +374,7 @@ static const char *hexDigit = "0123456789ABCDEF";
   bytes = [self bytes];
   len = [self length];
 
-  if (!theData)
+  if (theData == nil || [theData length] == 0)
     {
       return NSMakeRange(NSNotFound,0);
     }
@@ -410,7 +414,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 -(NSRange) rangeOfCString: (const char *) theCString
-		  options: (unsigned int) theOptions
+		  options: (NSUInteger) theOptions
 {
   return [self rangeOfCString: theCString 
 	       options: theOptions 
@@ -422,7 +426,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 -(NSRange) rangeOfCString: (const char *) theCString
-		  options: (unsigned int) theOptions
+		  options: (NSUInteger) theOptions
 		    range: (NSRange) theRange
 {
   const char *b, *bytes;
@@ -432,11 +436,22 @@ static const char *hexDigit = "0123456789ABCDEF";
     {
       return NSMakeRange(NSNotFound,0);
     }
+  if (theRange.location == NSNotFound)
+    {
+      NSDebugLog(@"rangeOfCString: invalid range location");
+      return NSMakeRange(NSNotFound,0);
+    }
   
   bytes = [self bytes];
   len = [self length];
   slen = (NSUInteger)strlen(theCString);
   
+  if (len < slen)
+    {
+      NSDebugLog(@"rangeOfCString: string too small to compare %@ to %s", self, theCString);
+      return NSMakeRange(NSNotFound,0);
+    }
+  
   b = bytes;
   
   if (len > theRange.location + theRange.length)
@@ -481,6 +496,9 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 - (NSData *) subdataFromIndex: (NSUInteger) theIndex
 {
+  if (theIndex > [self length])
+    return [NSData data];
+
   return [self subdataWithRange: NSMakeRange(theIndex, [self length] - theIndex)];
 }
 
@@ -501,25 +519,25 @@ static const char *hexDigit = "0123456789ABCDEF";
 {
   const char *bytes;
   NSUInteger i, len;
-  NSInteger j;
+  NSUInteger j;
   
   bytes = [self bytes];
   if (bytes == NULL)
-    return AUTORELEASE(RETAIN(self));
+    return [NSData data];
   
   len = [self length];
   if (len == 0)
-    return AUTORELEASE(RETAIN(self));
+    return [NSData data];
 
   for (i = 0; i < len && (bytes[i] == ' ' || bytes[i] == '\t'); i++) ;
-  for (j = len-1; j >= 0 && (bytes[j] == ' ' || bytes[i] == '\t'); j--) ;
+
+  // We found only spaces, no need to search from the end
+  if (i == len)
+    return [NSData data];
   
-  if (j <= i)
-    {
-      return AUTORELEASE(RETAIN(self));
-    }
+  for (j = len; j > 0 && (bytes[j-1] == ' ' || bytes[j-1] == '\t'); j--) ;
       
-  return [self subdataWithRange: NSMakeRange(i, j-i+1)];
+  return [self subdataWithRange: NSMakeRange(i, j-i)];
 }
 
 
@@ -607,8 +625,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 //
-#warning TODO / FIXME: rewrite to use NSUInteger and NSNotFound
-- (int) indexOfCharacter: (char) theCharacter
+- (NSUInteger) indexOfCharacter: (unichar) theCharacter
 {
   const char *b;
   NSUInteger i, len;
@@ -622,7 +639,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 	return i;
       }
   
-  return -1;
+  return NSNotFound;
 }
 
 
@@ -644,7 +661,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 
   slen = (NSUInteger)strlen(theCString);
   
-  if ( slen > len)
+  if ( slen == 0 ||  slen > len)
     {
       return NO;
     }
@@ -676,7 +693,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 
   slen = (NSUInteger)strlen(theCString);
 
-  if (slen > len) 
+  if (slen == 0 || slen > len) 
     {
       return NO;
     }
@@ -707,7 +724,7 @@ static const char *hexDigit = "0123456789ABCDEF";
   len = [self length];
   slen = (NSUInteger)strlen(theCString);
   
-  if ( slen > len)
+  if ( slen == 0 || slen > len)
     {
       return NO;
     }
@@ -738,7 +755,7 @@ static const char *hexDigit = "0123456789ABCDEF";
   len = [self length];
   slen = (NSUInteger)strlen(theCString);
 
-  if (slen > len) 
+  if (slen == 0 || slen > len) 
     {
       return NO;
     }  
@@ -895,7 +912,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 //
-- (NSData *) unwrapWithLimit: (int) theQuoteLimit
+- (NSData *) unwrapWithLimit: (NSUInteger) theQuoteLimit
 {
   NSMutableData *aMutableData, *lines;
   NSData *aLine;
@@ -984,10 +1001,10 @@ static const char *hexDigit = "0123456789ABCDEF";
 	  // The current line is flowed but has mis-matched quoting
 
 	  // We first append the previous paragraph to the buffer with the necessary quote characters
-	  if (quote_depth)
+	  if (quote_depth > 0)
 	    {
 	      [lines replaceBytesInRange: NSMakeRange(0, [lines length])
-		     withBytes: [[lines quoteWithLevel: quote_depth  wrappingLimit: theQuoteLimit] bytes]];
+		     withBytes: [[lines quoteWithLevel: (NSUInteger)quote_depth  wrappingLimit: theQuoteLimit] bytes]];
 	    }
 
 	  [aMutableData appendData: lines];
@@ -1005,11 +1022,11 @@ static const char *hexDigit = "0123456789ABCDEF";
 	  [lines appendData: aLine];
 	  
 	  // We add the necessary quote characters in the paragraph
-	  if (quote_depth)
+	  if (quote_depth > 0)
 	    {
 	      NSData *d;
 	      
-	      d = [lines quoteWithLevel: quote_depth   wrappingLimit: theQuoteLimit];
+	      d = [lines quoteWithLevel: (NSUInteger)quote_depth   wrappingLimit: theQuoteLimit];
 	      [lines replaceBytesInRange: NSMakeRange(0, [lines length])
 		     withBytes: [d bytes]  length: [d length]];
 	    }
@@ -1034,19 +1051,19 @@ static const char *hexDigit = "0123456789ABCDEF";
 	  // The line is fixed but has mis-matched quoting
 	  
 	  // We first append the previous paragraph (if any) to the buffer with the necessary quote characters
-	  if (quote_depth)
+	  if (quote_depth > 0)
 	    {
 	      [lines replaceBytesInRange: NSMakeRange(0, [lines length])
-		     withBytes: [[lines quoteWithLevel: quote_depth  wrappingLimit: theQuoteLimit] bytes]];
+		     withBytes: [[lines quoteWithLevel: (NSUInteger)quote_depth  wrappingLimit: theQuoteLimit] bytes]];
 	    }
 	  
 	  [aMutableData appendData: lines];
 	  [aMutableData appendCString: "\n"];
 
 	  // We append the fixed line to the buffer with the necessary quote characters
-	  if (line_quote_depth)
+	  if (line_quote_depth > 0)
 	    {
-	      aLine = [aLine quoteWithLevel: line_quote_depth  wrappingLimit: theQuoteLimit];
+	      aLine = [aLine quoteWithLevel: (NSUInteger)line_quote_depth  wrappingLimit: theQuoteLimit];
 	    }
 
 	  [aMutableData appendData: aLine];
@@ -1068,10 +1085,10 @@ static const char *hexDigit = "0123456789ABCDEF";
   // We must handle flowed lines that don't have a fixed line break at the end of the message
   if ([lines length])
     {
-      if (quote_depth)
+      if (quote_depth > 0)
 	{
 	  [lines replaceBytesInRange: NSMakeRange(0, [lines length])
-		 withBytes: [[lines quoteWithLevel: quote_depth  wrappingLimit: theQuoteLimit] bytes]];
+		 withBytes: [[lines quoteWithLevel: (NSUInteger)quote_depth  wrappingLimit: theQuoteLimit] bytes]];
 	}
 
       [aMutableData appendData: lines];
@@ -1087,14 +1104,14 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 //
-- (NSData *) wrapWithLimit: (int) theLimit
+- (NSData *) wrapWithLimit: (NSUInteger) theLimit
 {
   NSMutableData *aMutableData;
   NSData *aLine, *part;
   NSArray *lines;
   NSUInteger i, j, k;
-  int split;
-  int depth;
+  NSUInteger split;
+  NSUInteger depth;
 
   // We first verify if the string is valid
   if ([self length] == 0)
@@ -1282,8 +1299,8 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 //
-- (NSData *) quoteWithLevel: (int) theLevel
-	      wrappingLimit: (int) theLimit
+- (NSData *) quoteWithLevel: (NSUInteger) theLevel
+	      wrappingLimit: (NSUInteger) theLimit
 {
   NSMutableData *aMutableData, *aQuotePrefix;
   NSData *aData, *aLine;
@@ -1372,7 +1389,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 //
 - (void) insertCString: (const char *) theCString
-	       atIndex: (int) theIndex
+	       atIndex: (NSUInteger) theIndex
 {
   NSUInteger s_length, length;
 
@@ -1391,7 +1408,7 @@ static const char *hexDigit = "0123456789ABCDEF";
   length = [self length];
   
   // We insert at the beginning of the data
-  if (theIndex <= 0)
+  if (theIndex == 0)
     {
       NSMutableData *data;
       
@@ -1505,7 +1522,7 @@ static const char *hexDigit = "0123456789ABCDEF";
 //
 // C functions
 //
-int getValue(char c)
+int getValue(const unsigned char c)
 {
   if (c >= 'A' && c <= 'Z') return (c - 'A');
   if (c >= 'a' && c <= 'z') return (c - 'a' + 26);
@@ -1520,7 +1537,7 @@ int getValue(char c)
 //
 //
 //
-void nb64ChunkFor3Characters(char *buf, const char *inBuf, int theLength)
+void nb64ChunkFor3Characters(char *buf, const char *inBuf, unsigned theLength)
 {
   if (theLength >= 3)
     {
